/**
  ******************************************************************************
  * @file    Multi/Examples/MotionControl/IHM05A1_ExampleFor1Motor/Src/main.c
  * @author  IPC Rennes
  * @version V1.5.0
  * @date    June 1st, 2018
  * @brief   This example shows how to use 1 IHM05A1 expansion board
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2016 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/** @defgroup IHM05A1_Example_for_1_motor_device
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static volatile uint16_t gLastError;

/* Initialization parameters. */
l6208_Init_t initDeviceParameters =
{
  1500,            //Acceleration rate in step/s^2 or (1/16)th step/s^2 for microstep modes
  20,              //Acceleration current torque in % (from 0 to 100)
  1500,            //Deceleration rate in step/s^2 or (1/16)th step/s^2 for microstep modes
  20,              //Deceleration current torque in % (from 0 to 100)
  1500,            //Running speed in step/s or (1/16)th step/s for microstep modes
  10,              //Running current torque in % (from 0 to 100)
  5,               //Holding current torque in % (from 0 to 100)
  STEP_MODE_1_16,  //Step mode via enum motorStepMode_t
  FAST_DECAY,      //Decay mode via enum motorDecayMode_t
  0,               //Dwelling time in ms
  FALSE,           //Automatic HIZ STOP
  100000           //VREFA and VREFB PWM frequency (Hz)
};

/* Private function prototypes -----------------------------------------------*/
static void MyFlagInterruptHandler(void);
void ButtonHandler(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
  int32_t pos;
  uint16_t mySpeed;
  uint32_t freqPwm;

  /* STM32xx HAL library initialization */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

//----- Init of the Motor control library
  /* Set the L6208 library to use 1 device */
  BSP_MotorControl_SetNbDevices(BSP_MOTOR_CONTROL_BOARD_ID_L6208, 1);
  /* When BSP_MotorControl_Init is called with NULL pointer,                  */
  /* the L6208 parameters are set with the predefined values from file        */
  /* l6208_target_config.h, otherwise the parameters are set using the        */
  /* initDeviceParameters structure values.                                   */
  BSP_MotorControl_Init(BSP_MOTOR_CONTROL_BOARD_ID_L6208, NULL);
  //BSP_MotorControl_Init(BSP_MOTOR_CONTROL_BOARD_ID_L6208, &initDeviceParameters);

  /* Attach the function MyFlagInterruptHandler (defined below) to the flag interrupt */
  BSP_MotorControl_AttachFlagInterrupt(MyFlagInterruptHandler);

  /* Attach the function MyErrorHandler (defined below) to the error Handler*/
  BSP_MotorControl_AttachErrorHandler(MyErrorHandler);

//----- Configure Button for user interaction

   /* Set Systick Interrupt priority highest to ensure no lock by using HAL_Delay */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0x0, 0x0);

  /* Configure KEY Button */
  BSP_PB_Init(BUTTON_KEY, BUTTON_MODE_EXTI);

//----- Disable the power bridges after initialization of the L6208 device

  /* Disable the power bridges after initialization */
  BSP_MotorControl_CmdDisable(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Get the PWM frequency used for the VREFA and VREFB voltage generation

  freqPwm = BSP_MotorControl_GetBridgeInputPwmFreq(0);

//----- Set the PWM frequency used for the VREFA and VREFB voltage generation

  BSP_MotorControl_SetBridgeInputPwmFreq(0, freqPwm>>1);

//----- Move of 16000 microsteps in the FW direction

  /* Move device 16000 microsteps in the FORWARD direction */
  BSP_MotorControl_Move(0, FORWARD, 16000);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Move of 16000 microsteps in the BW direction

  /* Move device 16000 microsteps in the BACKWARD direction*/
  BSP_MotorControl_Move(0, BACKWARD, 16000);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos = BSP_MotorControl_GetPosition(0);

  /* Set the current position to be the Home position */
  BSP_MotorControl_SetHome(0, pos);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Go to position -6400

  /* Request to go to position -6400 */
  BSP_MotorControl_GoTo(0, -6400);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos = BSP_MotorControl_GetPosition(0);

  if (pos != -6400)
  {
    MyErrorHandler(L6208_ERROR_POSITION);
  }

    /* Get current position */
  pos = BSP_MotorControl_GetPosition(0);

  /* Set the current position to be the Mark position */
  BSP_MotorControl_SetMark(0, pos);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Go Home

  /* Request to go to Home */
  BSP_MotorControl_GoHome(0);
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos = BSP_MotorControl_GetPosition(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Go to position 6400

  /* Request to go to position 6400 */
  BSP_MotorControl_CmdGoToDir(0, FORWARD, 6400);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos = BSP_MotorControl_GetPosition(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Go Mark which was set previously after go to -6400

  /* Request to go to Mark position */
  BSP_MotorControl_GoMark(0);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos = BSP_MotorControl_GetPosition(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Run the motor BACKWARD

  /* Request to run BACKWARD */
   BSP_MotorControl_Run(0, BACKWARD);
   HAL_Delay(5000);

   /* Get current speed */
   mySpeed = BSP_MotorControl_GetCurrentSpeed(0);

//----- Increase the speed while running

  /* Increase speed to 2400 microstep/s */
  BSP_MotorControl_SetMaxSpeed(0, 2400);
  HAL_Delay(5000);

   /* Get current speed */
   mySpeed = BSP_MotorControl_GetCurrentSpeed(0);

//----- Decrease the speed while running

  /* Decrease speed to 1200 microstep/s */
  BSP_MotorControl_SetMaxSpeed(0, 1200);
  HAL_Delay(5000);

  /* Get current speed */
  mySpeed = BSP_MotorControl_GetCurrentSpeed(0);

//----- Increase acceleration while running

  /* Increase acceleration to 2000 microstep/s^2 */
  BSP_MotorControl_SetAcceleration(0, 2000);
  HAL_Delay(5000);

  /* Increase speed to 2400 microstep/s */
  BSP_MotorControl_SetMaxSpeed(0, 2400);
  HAL_Delay(5000);

  /* Get current speed */
  mySpeed = BSP_MotorControl_GetCurrentSpeed(0);

  if (mySpeed != 2400)
  {
    MyErrorHandler(L6208_ERROR_SPEED);
  }
//----- Increase deceleration while running

  /* Increase deceleration to 2000 microstep/s^2 */
  BSP_MotorControl_SetDeceleration(0, 2000);
  HAL_Delay(5000);

  /* Decrease speed to 1200 microstep/s */
  BSP_MotorControl_SetMaxSpeed(0, 1200);
  HAL_Delay(5000);

  /* Get current speed */
  mySpeed = BSP_MotorControl_GetCurrentSpeed(0);

//----- Soft stopped required while running

  /* Request soft stop */
  BSP_MotorControl_SoftStop(0);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Run stopped by hardstop

  /* Request to run in FORWARD direction */
  BSP_MotorControl_Run(0, FORWARD);
  HAL_Delay(5000);

  /* Request to immediatly stop */
  BSP_MotorControl_HardStop(0);
  BSP_MotorControl_WaitWhileActive(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- GOTO stopped by softstop

 /* Request to go to position 20000  */
  BSP_MotorControl_GoTo(0, 20000);
  HAL_Delay(5000);

  /* Request to perform a soft stop */
  BSP_MotorControl_SoftStop(0);
  BSP_MotorControl_WaitWhileActive(0);

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Change step mode to full step mode

  /* Select full step mode (normal mode) */
  BSP_MotorControl_SelectStepMode(0, STEP_MODE_FULL);

  /* Set speed, acceleration and deceleration to scale with normal mode */
  /* For normal mode and half step mode, these parameters are in steps/s or /s^2 */
  /* For microstep modes, these parameters are either in (1/16)step/s or /s^2 */
  BSP_MotorControl_SetMaxSpeed(0, BSP_MotorControl_GetMaxSpeed(0)>>4);
  BSP_MotorControl_SetMinSpeed(0, L6208_MIN_SPEED);
  BSP_MotorControl_SetAcceleration(0, BSP_MotorControl_GetAcceleration(0)>>4);
  BSP_MotorControl_SetDeceleration(0, BSP_MotorControl_GetDeceleration(0)>>4);

  /* Request to go position 200 (full steps) */
  BSP_MotorControl_GoTo(0, 200);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos =  BSP_MotorControl_GetPosition(0);

  if (pos != 200)
  {
    MyErrorHandler(L6208_ERROR_POSITION);
  }

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Change step mode to half step mode
  /* Select half step mode */
  BSP_MotorControl_SelectStepMode(0, STEP_MODE_HALF);

  /* Request to go position -400 (half steps) */
  BSP_MotorControl_GoTo(0, -400);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos =  BSP_MotorControl_GetPosition(0);

  if (pos != -400)
  {
    MyErrorHandler(L6208_ERROR_POSITION);
  }

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Change step mode 1/4 microstepping mode
  /* Select 1/4 step mode */
  BSP_MotorControl_SelectStepMode(0, STEP_MODE_1_4);

  /* Set speed, acceleration and deceleration to scale with microstep mode */
  /* For normal mode and half step mode, these parameters are in steps/s or /s^2 */
  /* For microstep modes, these parameters are either in (1/16)step/s or /s^2 */
  BSP_MotorControl_SetMaxSpeed(0, BSP_MotorControl_GetMaxSpeed(0)<<4);
  BSP_MotorControl_SetMinSpeed(0, L6208_MIN_SPEED);
  BSP_MotorControl_SetAcceleration(0, BSP_MotorControl_GetAcceleration(0)<<4);
  BSP_MotorControl_SetDeceleration(0, BSP_MotorControl_GetDeceleration(0)<<4);

  /* Request to go position 800 (quarter steps) */
  BSP_MotorControl_GoTo(0, 800);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos =  BSP_MotorControl_GetPosition(0);

  if (pos != 800)
  {
    MyErrorHandler(L6208_ERROR_POSITION);
  }

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Change step mode 1/8 microstepping mode
  /* Select 1/8 step mode */
  BSP_MotorControl_SelectStepMode(0, STEP_MODE_1_8);

  /* Request to go position -1600 (1/8th steps) */
  BSP_MotorControl_GoTo(0, -1600);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos =  BSP_MotorControl_GetPosition(0);

  if (pos != -1600)
  {
    MyErrorHandler(L6208_ERROR_POSITION);
  }

  /* Wait for 2 seconds */
  HAL_Delay(2000);

//----- Restore 1/16 microstepping mode

  /* Reset device to 1/16 microstepping mode */
  BSP_MotorControl_SelectStepMode(0, STEP_MODE_1_16);

  /* Set speed and acceleration at lowest values */
  BSP_MotorControl_SetMaxSpeed(0, L6208_MIN_SPEED);
  BSP_MotorControl_SetAcceleration(0, L6208_MIN_ACC_DEC_RATE);
  BSP_MotorControl_SetDeceleration(0, L6208_MIN_ACC_DEC_RATE);

  /* Move device 49 microsteps (1/16th steps) in the FORWARD direction*/
  BSP_MotorControl_Move(0, FORWARD, 49);

  /* Wait for the motor ends moving */
  BSP_MotorControl_WaitWhileActive(0);

  /* Get current position */
  pos =  BSP_MotorControl_GetPosition(0);

  if (pos != 49)
  {
    MyErrorHandler(L6208_ERROR_POSITION);
  }

  /* Set speed and acceleration from powerspin6208_target_config.h */
  BSP_MotorControl_SetMaxSpeed(0, L6208_CONF_PARAM_RUNNING_SPEED);
  BSP_MotorControl_SetAcceleration(0, L6208_CONF_PARAM_ACC_RATE);
  BSP_MotorControl_SetDeceleration(0, L6208_CONF_PARAM_DEC_RATE);

  /* Turn off power bridges when motor is stopped */
  BSP_MotorControl_SetStopMode(0, HIZ_MODE);

  /* Infinite loop */
  while(1)
  {
    /* Request to run */
    BSP_MotorControl_Run(0, BSP_MotorControl_GetDirection(0));
    HAL_Delay(5000);

    /* Request soft stop */
    BSP_MotorControl_SoftStop(0);
    HAL_Delay(2000);
  }
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);
  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL6;
  RCC_OscInitStruct.PLL.PLLDIV = RCC_PLL_DIV3;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief  This function is the User handler for the flag interrupt
  * @param  None
  * @retval None
  */
void MyFlagInterruptHandler(void)
{
  //When EN pin is forced low by a failure, configure the GPIO as an ouput low
  BSP_MotorControl_CmdDisable(0);
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  error number of the error
  * @retval None
  */
void MyErrorHandler(uint16_t error)
{
  /* Backup error number */
  gLastError = error;

  /* Infinite loop */
  while(1)
  {
  }
}

/**
  * @brief  This function is executed in case of button press and it changes the
  * current direction of the motor
  * @param  None
  * @retval None
  */
void ButtonHandler(void)
{
  if (BSP_MotorControl_GetDirection(0) != BACKWARD)
  {
    BSP_MotorControl_SetDirection(0, BACKWARD);
  }
  else
  {
    BSP_MotorControl_SetDirection(0, FORWARD);
  }
  /* Let 200 ms before clearing the IT for key debouncing */
  HAL_Delay(200);
  __HAL_GPIO_EXTI_CLEAR_IT(KEY_BUTTON_PIN);
  HAL_NVIC_ClearPendingIRQ(KEY_BUTTON_EXTI_IRQn);
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
